"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var helpers_1 = require("./helpers");
describe('Helpers', function () {
    describe('load()', function () {
        it('load() should return a promise', function () {
            var observer = helpers_1.load({ family: 'foo' }, 5000);
            expect(observer instanceof Promise).toBeTruthy();
        });
    });
    describe('dataWithLoadingFont()', function () {
        it('should map basic font data only', function () {
            var actual = helpers_1.dataWithLoadingFont({
                family: 'Foo',
                foo: 'bar',
                baz: 42,
            });
            expect(actual.foo).toBeUndefined();
            expect(actual.baz).toBeUndefined();
        });
        it('should map all font props', function () {
            var input = {
                family: 'Foo',
                weight: 300,
                stretch: 'expanded',
                style: 'oblique',
            };
            var expected = {
                family: 'Foo',
                weight: 300,
                stretch: 'expanded',
                style: 'oblique',
                styles: {},
                class: '',
            };
            expect(helpers_1.dataWithLoadingFont(input)).toEqual(expected);
        });
        it('should set initial class and styles', function () {
            var actual = helpers_1.dataWithLoadingFont({
                family: 'Foo',
                class: {
                    initial: 'system-font',
                    success: 'foo-font',
                },
                styles: {
                    initial: {
                        a: 1,
                    },
                    success: {
                        b: 1,
                    },
                },
            });
            expect(actual.class).toEqual('system-font');
            expect(actual.styles).toEqual({ a: 1 });
        });
    });
    describe('dataWithLoadedFont', function () {
        it('dataWithLoadedFont should use default values for class and styles props', function () {
            var input = {
                family: 'Foo',
            };
            var expected = {
                family: 'Foo',
                class: '',
                styles: {},
                timing: expect.any(Number),
            };
            expect(helpers_1.dataWithLoadedFont(input)).toEqual(expected);
        });
        it('dataWithLoadedFont should map full font data', function () {
            var input = {
                family: 'Foo',
                custom: 42,
                class: {
                    success: 'foo-font-loaded',
                },
                styles: {
                    success: {
                        fontFamily: 'Foo, monospace',
                    },
                },
            };
            var expected = {
                family: 'Foo',
                custom: 42,
                class: 'foo-font-loaded',
                styles: {
                    fontFamily: 'Foo, monospace',
                },
                timing: expect.any(Number),
            };
            expect(helpers_1.dataWithLoadedFont(input)).toEqual(expected);
        });
    });
    describe('dataWithFailedFont()', function () {
        it('should map fallback font data', function () {
            var input = {
                family: 'Foo',
                class: {
                    success: 'foo-font',
                    fallback: 'fallback-font',
                },
                styles: {
                    success: {
                        fontFamily: 'Foo, monospace',
                    },
                    fallback: {
                        fontFamily: 'Monaco, monospace',
                    },
                },
            };
            var expected = {
                family: 'Foo',
                class: 'fallback-font',
                styles: {
                    fontFamily: 'Monaco, monospace',
                },
                error: expect.any(Object),
            };
            expect(helpers_1.dataWithFailedFont(input)).toEqual(expected);
        });
        it('should use default fallback values', function () {
            var input = {
                family: 'Foo',
                class: {
                    success: 'bar-font',
                },
            };
            var expected = {
                family: 'Foo',
                class: '',
                styles: {},
                error: expect.any(Object),
            };
            expect(helpers_1.dataWithFailedFont(input)).toEqual(expected);
        });
        it('should use initial data if no fallback values provided', function () {
            var input = {
                family: 'Foo',
                class: {
                    initial: 'system-font',
                    success: 'foo-font',
                },
                styles: {
                    initial: {
                        fontFamily: 'Arial, sans-serif',
                    },
                    success: {
                        fontFamily: 'Foo, sans-serif',
                    },
                },
            };
            var expected = {
                family: 'Foo',
                class: 'system-font',
                styles: {
                    fontFamily: 'Arial, sans-serif',
                },
                error: expect.any(Error),
            };
            expect(helpers_1.dataWithFailedFont(input)).toEqual(expected);
        });
    });
});
//# sourceMappingURL=helpers.spec.js.map